# 常用小模块
import datetime
import time

from cffi.backend_ctypes import long
from pytz import timezone


def time_taken(end_time, start_time):
    time_spend = end_time - start_time
    m, s = divmod(time_spend, 60)
    h, m = divmod(m, 60)
    d, h = divmod(h, 24)
    return d, h, m, s


def get_date(days=1, str_format='%Y-%m-%d', time_zone='Asia/Shanghai', current_day=None):
    """
    基于今天的日期，返回指定间隔的日期字符串，
    :param days: 距离“今天”的日期，0表示今天，1表示昨天，依次类推
    :param str_format: 格式化后的字符串
    :param time_zone: 目标时间的时区，例如 Asia/Shanghai，America/New_York
    :param current_day: 当前日志，默认是字符串格式
    :return: 字符串格式的日期，例如 2022-12-20
    """
    # 获取当天时间——服务器时间
    if current_day is None:  # 如果不指定，则使用系统当前日期
        dt_loc_today = datetime.datetime.now()
    else:  # 否则使用指定的
        dt_loc_today = datetime.datetime.strptime(current_day, str_format)
    # 获取对应时区的时间——转换为目标时区时间
    utc8 = timezone(time_zone)
    dt_utc8 = dt_loc_today.astimezone(utc8)
    # 获取日期间隔的时间——基于目标时区时间
    the_day = dt_utc8 - datetime.timedelta(days=int(days))
    return the_day.strftime(str_format)


def get_current_timestamp(str_format='%Y%m%d%H%M%S'):
    return time.strftime(str_format, time.localtime(time.time()))


# 获得指定的时间戳
def get_timestamp(timestamp_str='', format='%Y-%m-%d %H:%M:%S'):
    tm_object = time.strptime(timestamp_str, format)
    return int(time.mktime(tm_object)*1000)



import mimetypes
import smtplib
from email import encoders
from email.header import Header
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formataddr


def _get_attach_msg(file_path):  #
    ctype, encoding = mimetypes.guess_type(file_path)  #
    if ctype is None or encoding is not None:  #
        ctype = 'application/octet-stream'  #
    maintype, subtype = ctype.split('/', 1)  #
    with open(file_path, 'rb') as fp:  #
        msg = MIMEBase(maintype, subtype)  #
        msg.set_payload(fp.read())  #
    encoders.encode_base64(msg)  #
    msg.add_header('Content-Disposition', 'attachment', filename=file_path.split('/')[-1])  #
    return msg


def send_mail(task_type, date, from_user_name, from_user, host, port, user,
              passwd, receivers, html_message, log_file=None):
    # 全局变量
    mail_subject = f'[{date}]UFS-{task_type}Azure Events数据更新预警-[{html_message}]'
    # 构建正文信息
    message = f""" <h1> {mail_subject} </h1><hr /><br>
                    UFS Azure Databricks环境下，tracking.events更新出现小时数据为空的情况，信息如下：<br>
                    {html_message}
                  """

    # 构建邮件附件
    msg = MIMEMultipart()  #`
    msg['From'] = formataddr((from_user_name, from_user))  #
    msg['Subject'] = Header(mail_subject, 'utf-8').encode()  #
    msg.attach(MIMEText(message, 'html', 'utf-8'))  #
    if log_file is not None:
        msg.attach(_get_attach_msg(log_file))  # 添加log附件
    # 发送邮件
    smtp = smtplib.SMTP_SSL(host, port)  # 注意这里使用SSL方式，下面的端口使用的是 465
    smtp.ehlo()
    smtp.login(user, passwd)  #
    smtp.sendmail(from_user, receivers, msg.as_string())  #



# -*- coding:utf-8 -*-

import logging
import os
import sys
import traceback
import datetime
from logging.handlers import TimedRotatingFileHandler


def check_path(file_path):
    """检查输出目录是否存在，如果不存在则新建目录
    如果path_type=='file'或'dir'

    注意：使用os.path.isdir 或 os.path.isfile 时，如果路径不存在，则返回False
    所以：必须使用绝对路径
    """
    split_info = os.path.splitext(os.path.abspath(file_path))
    if split_info[1] == '':  # 如果是路径，没有文件扩展名
        path = file_path
    else:  # 先拆分出路径，再判断
        path, _ = os.path.split(os.path.abspath(file_path))
    if not os.path.exists(path):
        os.makedirs(path)  # 注意使用多级目录方法


class LOG:
    """
    输出log日志到指定目录

    参数
    ----
    log_path: log日志所在的完整路径，包含log文件

    示例
    ----

    >>> from utils.op_log import LOG
    >>> file_path = '../log/20190306164611.log'
    >>> log_ins = LOG(file_path)
    >>> log_ins.info('this is a info 1')
    >>> log_ins.error('this is a info 1')
    >>> log_ins.warning('this is a info 1')
    >>> log_ins.close()

    备注
    ---
    file_path必须是包含log文件名的路径，可以是绝对路径，也可以是相对路径。

    不同的信息，调用不同的方法，例如：一般常用的是info方法，在Exception是调用error，
    在警告时调用warning方法。

    实例化对象的最后，务必调用close方法关闭对象。否则会导致不同的信息，写入相同的log
    文件中。

    """

    def __init__(self, log_path, if_print_in_console=True):
        file_path, _ = os.path.split(log_path)
        check_path(file_path)  # 检查目录是否存在
        # 获取logger实例
        self.logger = logging.getLogger()
        # 设置日志级别
        self.logger.setLevel(logging.INFO)
        # 文件日志
        self.file_handler = logging.FileHandler(log_path)
        # 消息记录格式
        formatter = logging.Formatter(
            '%(asctime)s - %(levelname)s - %(message)s')
        self.file_handler.setFormatter(formatter)
        # 控制台日志
        self.console_handler = logging.StreamHandler(sys.stdout)
        self.console_handler.formatter = formatter  # 也可以直接给formatter赋值
        # 为logger添加的日志处理器
        self.logger.addHandler(self.file_handler)
        self.logger.addHandler(self.console_handler)
        # 控制是否在控制台打印输出
        self.if_print_in_console = if_print_in_console

    def info(self, info):
        self.logger.info(info)
        if not self.if_print_in_console:
            self.logger.removeHandler(self.console_handler)

    def error(self, info):
        self.logger.error(f'Error: {info}, here are details: {traceback.format_exc()} .\n')
        if not self.if_print_in_console:
            self.logger.removeHandler(self.console_handler)

    def debug(self, message):
        self.logger.debug(message)

    def warning(self, message):
        self.logger.warning(message)

    def critical(self, message):
        self.logger.critical(message)

    def remove(self):
        self.logger.removeHandler(self.file_handler)
        self.logger.removeHandler(self.console_handler)

    def close(self):
        self.file_handler.close()
        self.console_handler.close()


# 适用于API等长期在线的日志
def get_logger(log_path):
    check_path(log_path)  # 检查目录是否存在
    # 获取logger实例
    log = logging.getLogger()
    # 设置日志级别
    log.setLevel(logging.INFO)
    # 消息记录格式
    formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
    # 各种日志处理器
    file_handler = logging.FileHandler(log_path)  # 文件日志
    console_handler = logging.StreamHandler(sys.stdout)  # 控制台日志
    split_handler = TimedRotatingFileHandler(log_path, when='midnight', interval=1,
                                             backupCount=7, encoding='utf-8',
                                             atTime=datetime.time(0, 0, 0, 0))  # 日期分割
    # 添加格式
    handles = [file_handler, console_handler, split_handler]
    for i in handles:
        i.setFormatter(formatter)
        log.addHandler(i)
    return log
